﻿//*************************************************************************************************
// ExtractionRuleInnerText.cs
// Owner: Herodotos Herodotou (t-herohe)
//
// An extraction rule used to extract the inner text from an html tag
//
// Copyright(c) Microsoft Corporation, 2008
//*************************************************************************************************

using System;
using System.ComponentModel;
using System.Globalization;
using WebAndLoadTestPluginsAndRules.Properties;

using Microsoft.VisualStudio.TestTools.WebTesting;

namespace WebAndLoadTestPluginsAndRules
{
    [LocalizableDisplayName("ValidationRuleNameTagInnerText")]
    [LocalizableDescription("ValidationRuleDescriptionTagInnerText")]
    public class ExtractionRuleInnerText : ExtractionRule
    {
        // ****************************************************************************
        // Overriden method
        // ****************************************************************************

        /// <summary>
        /// Extract the Inner Text of a particular html tag
        /// </summary>
        public override void Extract(object sender, ExtractionEventArgs e)
        {
            int i = 0;
            HtmlTagInnerTextParser parser = new HtmlTagInnerTextParser(e.Response.BodyString);
            foreach (InnerText innerText in parser.GetInnerTextForHtmlTags(TagName, 
                AttributeName, AttributeValue, HasClosingTags, RemoveInnerTags, CollapseWhiteSpace))
            {
                if (Index == -1 || Index == i)
                {
                    e.WebTest.Context[this.ContextParameterName] = innerText.Value;
                    e.Success = true;
                    return;
                }
                ++i;
            }

            // Unable to find the tag
            e.Success = false;
            if (!e.Response.IsHtml)
            {
                e.Message = string.Format(CultureInfo.InvariantCulture,
                    Resources.ErrorResponseNotHtmlForExtractionRules);
            }
            else
            {
                e.Message = string.Format(CultureInfo.InvariantCulture,
                    Resources.ExtractionErrorTagNotFound, TagName);
            }
        }


        // ****************************************************************************
        // Public Properties
        // ****************************************************************************

        [LocalizableDisplayName("PropertyNameTagName")]
        [LocalizableDescription("PropertyDescriptionTagName")]
        public string TagName
        {
            get { return m_tagName; }
            set { m_tagName = value; }
        }

        [LocalizableDisplayName("PropertyNameAttributeName")]
        [LocalizableDescription("PropertyDescriptionAttributeName")]
        public string AttributeName
        {
            get { return m_attributeName; }
            set { m_attributeName = (value.Length == 0) ? null : value; }
        }

        [LocalizableDisplayName("PropertyNameAttributeValue")]
        [LocalizableDescription("PropertyDescriptionAttributeValue")]
        public string AttributeValue
        {
            get { return m_attributeValue; }
            set { m_attributeValue = (value.Length == 0) ? null : value; }
        }

        [LocalizableDisplayName("PropertyNameRemoveInnerTags")]
        [LocalizableDescription("PropertyDescriptionRemoveInnerTags")]
        [DefaultValue(true)]
        public bool RemoveInnerTags
        {
            get { return m_removeInnerTags; }
            set { m_removeInnerTags = value; }
        }

        [LocalizableDisplayName("PropertyNameClosingTag")]
        [LocalizableDescription("PropertyDescriptionClosingTag")]
        [DefaultValue(true)]
        public bool HasClosingTags
        {
            get { return m_hasClosingTag; }
            set { m_hasClosingTag = value; }
        }

        [LocalizableDisplayName("PropertyNameCollapseExtraWhiteSpace")]
        [LocalizableDescription("PropertyDescriptionCollapseExtraWhiteSpace")]
        [DefaultValue(true)]
        public bool CollapseWhiteSpace
        {
            get { return m_collapseWhiteSpace; }
            set { m_collapseWhiteSpace = value; }
        }

        [LocalizableDisplayName("PropertyNameIndex")]
        [LocalizableDescription("PropertyDescriptionIndex")]
        [DefaultValue(-1)]
        public int Index
        {
            get { return m_index; }
            set { m_index = value; }
        }


        // ************************************************************************
        // Private Member Variables.
        // ************************************************************************

        private string m_tagName;
        private string m_attributeName;
        private string m_attributeValue;

        private bool m_hasClosingTag;
        private bool m_removeInnerTags;
        private bool m_collapseWhiteSpace;
        private int m_index;
    }
}
