﻿//*************************************************************************************************
// ValidationRuleSelectTag.cs
// Owner: Herodotos Herodotou (t-herohe)
//
// A validation rule used to validate the selected option from a 'select' html tag
//
// Copyright(c) Microsoft Corporation, 2008
//*************************************************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using WebAndLoadTestPluginsAndRules.Properties;

using Microsoft.VisualStudio.TestTools.WebTesting;

namespace WebAndLoadTestPluginsAndRules
{
    [LocalizableDisplayName("ValidationRuleNameSelectStatement")]
    [LocalizableDescription("ValidationRuleDescriptionSelectStatement")]
    public class ValidationRuleSelectTag : ValidationRule
    {
        // ****************************************************************************
        // Overriden method
        // ****************************************************************************

        /// <summary>
        /// Validate the selected option of a particular html select tag
        /// </summary>
        public override void Validate(object sender, ValidationEventArgs e)
        {
            int i = 0;
            foreach (string selected in new SelectTagParser(e.Response.BodyString).GetAllSelectedOptions(SelectTagName))
            {
                if (Index == -1 || Index == i)
                {
                    if (IgnoreCase == true)
                    {
                        e.IsValid = string.Equals(selected, ExpectedSelectedOption, StringComparison.OrdinalIgnoreCase);
                    }
                    else
                    {
                        e.IsValid = string.Equals(selected, ExpectedSelectedOption, StringComparison.Ordinal);
                    }

                    if (e.IsValid)
                    {
                        return;
                    }
                }
                ++i;
            }

            // Unable to find the tag
            e.IsValid = false;
            if (!e.Response.IsHtml)
            {
                e.Message = string.Format(CultureInfo.InvariantCulture,
                    Resources.ErrorResponseNotHtmlForValidationRules);
            }
            else
            {
                e.Message = string.Format(CultureInfo.InvariantCulture,
                    Resources.ValidationErrorSelectedOptionNotFound, ExpectedSelectedOption);
            }
        }


        // ****************************************************************************
        // Public Properties
        // ****************************************************************************

        [LocalizableDisplayName("PropertyNameSelectTagName")]
        [LocalizableDescription("PropertyDescriptionSelectTagName")]
        public string SelectTagName
        {
            get { return m_selectTagName; }
            set { m_selectTagName = (value.Length == 0) ? null : value; }
        }

        [LocalizableDisplayName("PropertyNameExpectedSelectedValue")]
        [LocalizableDescription("PropertyDescriptionExpectedSelectedValue")]
        public string ExpectedSelectedOption
        {
            get { return m_expectedSelectedOption; }
            set { m_expectedSelectedOption = value; }
        }

        [LocalizableDisplayName("PropertyNameIndex")]
        [LocalizableDescription("PropertyDescriptionIndex")]
        [DefaultValue(-1)]
        public int Index
        {
            get { return m_index; }
            set { m_index = value; }
        }

        [LocalizableDisplayName("PropertyNameIgnoreCase")]
        [LocalizableDescription("PropertyDescriptionIgnoreCase")]
        [DefaultValue(true)]
        public bool IgnoreCase
        {
            get { return m_ignoreCase; }
            set { m_ignoreCase = value; }
        }


        // ****************************************************************************
        // Private Member Variables
        // ****************************************************************************

        private string m_selectTagName;
        private string m_expectedSelectedOption;
        private int m_index;
        private bool m_ignoreCase;
    }
}
